/* Save contents of a GZZ window as IFF picture                               */

/*----------------------------------------------------------------------------*/
/* Systen Includes                                                            */
/*----------------------------------------------------------------------------*/

	#include <proto/exec.h>
	#include <proto/dos.h>
	#include <proto/graphics.h>
	#include <proto/intuition.h>
	#include <proto/datatypes.h>
	#include <datatypes/pictureclass.h>
	#include <cybergraphx/cybergraphics.h>

	#include <string.h>

/*----------------------------------------------------------------------------*/
/* Draw a nice pattern                                                        */
/*----------------------------------------------------------------------------*/

	void draw_something (struct RastPort *rp,long w,long h)

	{
	UBYTE c = 0;
	long i;

	w --;
	h --;

	for (i = 0; i <= w; i += 5)
		{
		SetAPen (rp,c);
		Move (rp,i,0);
		Draw (rp,w-i,h);
		c ++;
		}

	for (i = 0; i <= h; i += 5)
		{
		SetAPen (rp,c);
		Move (rp,w,i);
		Draw (rp,0,h-i);
		c ++;
		}
	}

/*----------------------------------------------------------------------------*/
/* Alloc a new bitmap and copy the window contents into it                    */
/*----------------------------------------------------------------------------*/

	struct BitMap *cut_gzz_area (struct Window *win)

	{
	long w;
	long h;
	long d;
	struct BitMap *winbm = win->RPort->BitMap;
	struct BitMap *bm;
	struct RastPort rp;

	w = win->GZZWidth;
	h = win->GZZHeight;
	d = GetBitMapAttr(winbm,BMA_DEPTH);

	if (d > 8)
		bm = AllocBitMap (w,h,24,BMF_MINPLANES|BMF_SPECIALFMT|SHIFT_PIXFMT(PIXFMT_RGB24),NULL);
	else
		bm = AllocBitMap (w,h,d,0,winbm);

	if (bm)
		{
		memset (&rp,0,sizeof(struct RastPort));
		InitRastPort (&rp);
		rp.BitMap = bm;
		ClipBlit (win->RPort,0,0,&rp,0,0,w,h,0xc0);
		}

	return (bm);
	}

/*----------------------------------------------------------------------------*/
/* Save the inner contents of a GZZ window as IFF picture using datatypes     */
/*----------------------------------------------------------------------------*/

	BOOL save_window_contents (char *name,struct Window *win)

	{
	struct BitMap *bm;
	Object *o;
	struct BitMapHeader *bmhd;
	UBYTE *cmap;
	ULONG *cregs;
	long i;
	long w,h,d;
	long ncols;
	BPTR fhand;
	struct ViewPort *vp;
	BOOL ok = FALSE;

	if (bm = cut_gzz_area (win))
		{
		w = win->GZZWidth;
		h = win->GZZHeight;
		d = GetBitMapAttr(bm,BMA_DEPTH);

		ncols = (d > 8 ? 0 : 1 << d);

		vp = &(win->WScreen->ViewPort);

		if (o = NewDTObject (NULL,
				DTA_SourceType,DTST_RAM,
				DTA_GroupID,GID_PICTURE,
				PDTA_BitMap,bm,
				PDTA_ModeID,GetVPModeID(vp),
				(ncols ? PDTA_NumColors : PDTA_DestMode),(ncols ? ncols : PMODE_V43),
				TAG_END))
			{
			GetDTAttrs (o,PDTA_BitMapHeader,&bmhd,TAG_END);

			bmhd->bmh_Width  = w;
			bmhd->bmh_Height = h;
			bmhd->bmh_Depth  = d;
			bmhd->bmh_XAspect = 22;
			bmhd->bmh_YAspect = 22;
			bmhd->bmh_PageWidth = (w <= 320 ? 320 : w <= 640 ? 640 : w <= 1024 ? 1024 : w <= 1280 ? 1280 : 1600);
			bmhd->bmh_PageHeight = bmhd->bmh_PageWidth * 3 / 4;

			if (ncols)
				{
				GetDTAttrs (o,PDTA_ColorRegisters,&cmap,PDTA_CRegs,&cregs,TAG_END);
				GetRGB32 (vp->ColorMap,0,ncols,cregs);
				for (i = 3*ncols; i; i--)
					*cmap++ = (*cregs++) >> 24;
				}

			if (fhand = Open (name,MODE_NEWFILE))
				{
				i = DoDTMethod (o,NULL,NULL,DTM_WRITE,NULL,fhand,DTWM_IFF,NULL);
				Close (fhand);
				if (i)
					ok = TRUE;
				else
					DeleteFile (name);
				}

			DisposeDTObject (o);
			}
		else
			FreeBitMap (bm);
		}

	return (ok);
	}

/*----------------------------------------------------------------------------*/
/* Main program: open a window, draw a pattern and save it to a file          */
/*----------------------------------------------------------------------------*/

	int main (void)
	{
	struct RDArgs *rdargs;
	static struct {
		char *name;
		long *width;
		long *height;
		char *pubscreen;
		} args = {"ram:pic.iff",NULL,NULL,NULL};

	struct Screen *scr;
	struct Window *win;
	struct Message *msg;

	long winw = 400;
	long winh = 300;

	if (rdargs = ReadArgs ("NAME,WIDTH/N,HEIGHT/N,PUBSCREEN/K",(APTR)&args,NULL))
		{
		if (args.width)  winw = *args.width;
		if (args.height) winh = *args.height;

		if (scr = LockPubScreen (args.pubscreen))
			{
			if (win = OpenWindowTags (NULL,
					WA_CustomScreen,scr,
					WA_Width,winw,
					WA_Height,winh,
					WA_Left,(scr->Width - winw) >> 1,
					WA_Top,(scr->Height - winh) >> 1,
					WA_Flags,WFLG_CLOSEGADGET|WFLG_DRAGBAR|WFLG_DEPTHGADGET|WFLG_GIMMEZEROZERO|WFLG_ACTIVATE,
					WA_IDCMP,IDCMP_CLOSEWINDOW|IDCMP_VANILLAKEY,
					TAG_END))
				{

				draw_something (win->RPort,win->GZZWidth,win->GZZHeight);

				if (save_window_contents (args.name,win))
					Printf ("save successful\n");
				else
					Printf ("save failed: %s\n",GetDTString(IoErr()));

				DisplayBeep (scr);

				WaitPort (win->UserPort);
				while (msg = GetMsg (win->UserPort))
					ReplyMsg (msg);

				CloseWindow (win);
				}

			UnlockPubScreen (NULL,scr);
			}

		FreeArgs (rdargs);
		}

	PrintFault (IoErr(),NULL);
	return (0);
	}

/*----------------------------------------------------------------------------*/
/* End of the source code                                                     */
/*----------------------------------------------------------------------------*/
